"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addFixturePackage = addFixturePackage;
exports.cleanup = cleanup;
exports.createModularTestContext = createModularTestContext;
exports.mockInstallTemplate = mockInstallTemplate;
exports.modular = modular;
exports.runLocalModular = runLocalModular;

var _rimraf2 = _interopRequireDefault(require("rimraf"));

var _execa = _interopRequireDefault(require("execa"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var tmp = _interopRequireWildcard(require("tmp"));

var _util = require("util");

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var _fs = require("fs");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const rimraf = (0, _util.promisify)(_rimraf2.default);
const modularRoot = (0, _getModularRoot.default)();

function modular(str, opts = {}) {
  return (0, _execa.default)('yarnpkg', ['modular', ...str.split(' ')], {
    cwd: modularRoot,
    cleanup: true,
    ...opts
  });
}

async function cleanup(packageNames) {
  const packagesPath = _path.default.join(modularRoot, 'packages');

  const distPath = _path.default.join(modularRoot, 'dist');

  for (const packageName of packageNames) {
    await rimraf(_path.default.join(packagesPath, packageName));
    await rimraf(_path.default.join(distPath, packageName));
  } // run yarn so yarn.lock gets reset


  await (0, _execa.default)('yarnpkg', ['--silent'], {
    cwd: modularRoot
  });
}

async function addFixturePackage(name, options = {
  copy: true
}) {
  const packageSrcDir = _path.default.join(modularRoot, 'packages', name, 'src');

  await modular(`add ${name} --unstable-type package`, {
    stdio: 'inherit'
  });
  await _fsExtra.default.emptyDir(packageSrcDir);

  if (options.copy) {
    await _fsExtra.default.copy(_path.default.join(__dirname, '..', '__tests__', '__fixtures__', 'packages', name), packageSrcDir);
  }
}
/**
 * Creates a temporary Modular repo test environment with a default package.json and an empty packages workspace.
 * Modular node_modules is symlinked in the parent directory to provide all required dependencies without installing them.
 *
 * Use to create a clean context to run tests in where you can call `yarn modular` commands
 * without having to install dependencies nor tests within the main Modular repo.
 * @returns Path to temporary directory
 */


function createModularTestContext() {
  // Modular node_modules are copied in the parent folder
  const tempDir = tmp.dirSync().name;

  _fsExtra.default.symlinkSync(_path.default.join(modularRoot, 'node_modules'), _path.default.join(tempDir, 'node_modules')); // Actual mock modular repo is nested in the temp directory so that it can have its own clean node_modules


  const tempModularRepo = _path.default.join(tempDir, 'temp_repo');

  (0, _fs.mkdirSync)(tempModularRepo); // Create mock modular package.json

  const packageJson = {
    name: 'temp',
    license: 'MIT',
    private: true,
    modular: {
      type: 'root'
    },
    workspaces: ['packages/**', 'templates/**'],
    scripts: {
      modular: `ts-node ${modularRoot}/packages/modular-scripts/src/cli.ts`
    },
    browserslist: {
      production: ['>0.2%', 'not dead', 'not op_mini all'],
      development: ['last 1 chrome version', 'last 1 firefox version', 'last 1 safari version']
    }
  };

  _fsExtra.default.writeJSONSync(_path.default.join(tempModularRepo, 'package.json'), packageJson, {
    spaces: 2
  });

  return tempModularRepo;
}
/**
 * Mocks the installation of a template in the provided modular repo to avoid running `yarn` or installing it from a registry.
 *
 * Copies the template into a Templates workspace and symlinks them in node_modules so that it can be picked up by modular add.
 * @param templatePath Path to directory containing templates
 * @param modularRepo Modular repo in which to install them
 */


function mockInstallTemplate(templatePath, modularRepo) {
  const tempTemplatesPath = _path.default.join(modularRepo, 'templates');

  const templateName = _path.default.basename(templatePath); // Copy the template into the temporary repo


  _fsExtra.default.copySync(templatePath, _path.default.join(tempTemplatesPath, templateName)); // Symlink the templates into node_modules


  const tempRepoNodeModules = _path.default.join(modularRepo, 'node_modules');

  if (!_fsExtra.default.existsSync(tempRepoNodeModules)) _fsExtra.default.mkdirSync(tempRepoNodeModules);

  _fsExtra.default.symlinkSync(_path.default.join(tempTemplatesPath, templateName), _path.default.join(tempRepoNodeModules, templateName));
}
/**
 * Run the Modular cli present at the root path `modularFolder` with a working directory of `cwd`
 * passing an array of `modularArguments`
 *
 * Useful to run modular from source in a different directory (usually a temp directory created for tests)
 *
 * @param modularToRun The root modular repo folder containing the modular-scripts to run
 * @param cwd The target working directory where we want to run Modular
 * @param modularArguments A list of command-line arguments
 */


function runLocalModular(modularToRun, cwd, modularArguments) {
  return _execa.default.sync(_path.default.join(modularToRun, '/node_modules/.bin/ts-node'), [_path.default.join(modularToRun, '/packages/modular-scripts/src/cli.ts'), ...modularArguments], {
    cwd,
    env: { ...process.env,
      CI: 'true'
    }
  });
}
//# sourceMappingURL=utils.js.map